﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing.Printing;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Forms;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace HelloWorld
{
    /// <summary>
    /// Interaction logic for PrintWin.xaml
    /// </summary>
    public partial class PrintWin : Window
    {
        public PrintWin()
        {
            InitializeComponent();
        }

        private void PrintWin_OnLoaded(object sender, RoutedEventArgs e)
        {
            SetupMapWithBlankPage();
            AddMapLayer();

            AddLegendLayer("111111111111111111111111111111111111111111111", false);
            Map1.Refresh();
           
        }


        void SetupMapWithBlankPage()
        {


            Map1.MapUnit = GeographyUnit.Meter;
            Map1.ZoomLevelSet = new PrinterZoomLevelSet(Map1.MapUnit,
                PrinterHelper.GetPointsPerGeographyUnit(Map1.MapUnit));
            Map1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.LightGray);
            var printerOverlay = new PrinterInteractiveOverlay();
            Map1.InteractiveOverlays.Add("PrintPreviewOverlay", printerOverlay);
            Map1.InteractiveOverlays.MoveToBottom("PrintPreviewOverlay");
            var pagePrinterLayer = new PagePrinterLayer(PrinterPageSize.AnsiA, PrinterOrientation.Portrait);
            pagePrinterLayer.Open();
            printerOverlay.PrinterLayers.Add("PageLayer", pagePrinterLayer);
            Map1.CurrentExtent = AreaBaseShape.ScaleUp(pagePrinterLayer.GetPosition(), 10).GetBoundingBox();
            Map1.MinimumScale = Map1.ZoomLevelSet.ZoomLevel20.Scale;

        }
        void AddMapLayer()
        {
            var mapPrinterLayer = new MapPrinterLayer
            {
                MapUnit = GeographyUnit.Meter,
                BackgroundMask = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black, 1))
            };

            mapPrinterLayer.Open();

            RectangleShape pageBoundingbox = GetPageBoundingBox(PrintingUnit.Inch);
            mapPrinterLayer.SetPosition(8, 7, pageBoundingbox.GetCenterPoint().X, pageBoundingbox.GetCenterPoint().Y + 1, PrintingUnit.Inch);



            var layer = new ShapeFileFeatureLayer(@"ShpFiles\Cardiff_Roads_polyline.shp");

           
                layer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(GetThinkGeoLineStyle());
                

          
            layer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;


            mapPrinterLayer.Layers.Add(layer);
            layer.Open();
            RectangleShape rectangleShape1 = layer.GetBoundingBox();
            layer.Close();


            var lst = new List<RectangleShape>();
            lst.Add(rectangleShape1);


            mapPrinterLayer.MapExtent = ExtentHelper.GetBoundingBoxOfItems(lst);



            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            printerInteractiveOverlay.PrinterLayers.Remove("MapLayer");
            printerInteractiveOverlay.PrinterLayers.Add("MapLayer", mapPrinterLayer);



        }
        RectangleShape GetPageBoundingBox(PrintingUnit unit)
        {
            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = (PagePrinterLayer)printerInteractiveOverlay.PrinterLayers["PageLayer"];
            return pagePrinterLayer.GetPosition(unit);
        }
        static LineStyle GetThinkGeoLineStyle()
        {
            return LineStyles.CreateSimpleLineStyle(
                  GeoColor.SimpleColors.Red,
                   2,
                   LineDashStyle.Solid,
                  false);
        }
        void AddLegendLayer(string legendText, bool? transparentBackground)
        {

            var legendAdornmentLayer = new LegendAdornmentLayer() {Height = 100};

            legendAdornmentLayer.LegendItems.Add(new LegendItem()
            {
                ImageStyle = new AreaStyle(new GeoSolidBrush(GeoColor.FromArgb(170, GeoColor.StandardColors.Green))),
                TextStyle = new TextStyle(legendText, new GeoFont("Arial", 8), new GeoSolidBrush(GeoColor.SimpleColors.Black))


            });



            var legendPrinterLayer = new LegendPrinterLayer(legendAdornmentLayer)
            {
                BackgroundMask = AreaStyles.CreateSimpleAreaStyle(transparentBackground == true ? GeoColor.SimpleColors.Transparent : new GeoColor(255, 230, 230, 230),
                    GeoColor.SimpleColors.Black, 1)
            };
            legendPrinterLayer.SetPosition(2, 1, -2.9, 3.9, PrintingUnit.Inch);



            var printerInteractiveOverlay = (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            if (!printerInteractiveOverlay.PrinterLayers.Contains("LegendPrinterLayer"))
                printerInteractiveOverlay.PrinterLayers.Add("LegendPrinterLayer", legendPrinterLayer);
            else
                printerInteractiveOverlay.PrinterLayers["LegendPrinterLayer"] = legendPrinterLayer;


        }
        static PaperSize GetPrintPreviewPaperSize(PagePrinterLayer pagePrinterLayer)
        {
            var printPreviewPaperSize = new PaperSize("AnsiA", 850, 1100);
            switch (pagePrinterLayer.PageSize)
            {
                case PrinterPageSize.AnsiA:
                    printPreviewPaperSize = new PaperSize("AnsiA", 850, 1100);
                    break;

                case PrinterPageSize.AnsiB:
                    printPreviewPaperSize = new PaperSize("AnsiB", 1100, 1700);
                    break;

                case PrinterPageSize.AnsiC:
                    printPreviewPaperSize = new PaperSize("AnsiC", 1700, 2200);
                    break;

                case PrinterPageSize.AnsiD:
                    printPreviewPaperSize = new PaperSize("AnsiD", 2200, 3400);
                    break;

                case PrinterPageSize.AnsiE:
                    printPreviewPaperSize = new PaperSize("AnsiE", 3400, 4400);
                    break;

                case PrinterPageSize.Custom:
                    printPreviewPaperSize = new PaperSize("Custom Size", (int)pagePrinterLayer.CustomWidth,
                        (int)pagePrinterLayer.CustomHeight);
                    break;
            }

            return printPreviewPaperSize;
        }



        private void BtnPrintPreview_OnClick(object sender, RoutedEventArgs e)
        {
            var printerInteractiveOverlay =
             (PrinterInteractiveOverlay)Map1.InteractiveOverlays["PrintPreviewOverlay"];
            var pagePrinterLayer = printerInteractiveOverlay.PrinterLayers["PageLayer"] as PagePrinterLayer;

            var printDocument = new PrintDocument
            {
                DefaultPageSettings =
                {
                    Landscape = pagePrinterLayer != null && pagePrinterLayer.Orientation == PrinterOrientation.Landscape,
                    PaperSize = GetPrintPreviewPaperSize(pagePrinterLayer)
                }
            };

            var printerGeoCanvas = new PrinterGeoCanvas { DrawingArea = printDocument.DefaultPageSettings.Bounds };
            if (pagePrinterLayer != null)
            {
                printerGeoCanvas.BeginDrawing(printDocument, pagePrinterLayer.GetBoundingBox(), Map1.MapUnit);
                printerGeoCanvas.Clear(new GeoSolidBrush(GeoColor.StandardColors.WhiteSmoke));

            }

            var labelsInAllLayers = new Collection<SimpleCandidate>();

            foreach (PrinterLayer printerLayer in printerInteractiveOverlay.PrinterLayers)
            {
                printerLayer.IsDrawing = true;
                if (!(printerLayer is PagePrinterLayer))
                    printerLayer.Draw(printerGeoCanvas, labelsInAllLayers);
                printerLayer.IsDrawing = false;
            }

            printerGeoCanvas.Flush();

            var printPreviewDialog = new PrintPreviewDialog { Document = printDocument };
            printPreviewDialog.ShowDialog();
        }
    }
}
